from pyhanko.sign import signers, timestamps
from certs_encoded import certs_encoded, timestamp_encoded
import asyncio
from asn1crypto import x509, tsp
from base64 import b64decode
from pyhanko_certvalidator.registry import SimpleCertificateStore
from pyhanko_certvalidator._asyncio_compat import to_thread

data_digest = b"aaaaaaaabbbbbbbbccccccccdddddddd"
signature_value = b"aaaaaaaabbbbbbbbccccccccdddddddd" * 16
certs = [x509.Certificate.load(b64decode(cert)) for cert in certs_encoded]
cert_registry = SimpleCertificateStore()
cert_registry.register_multiple(set(certs))
signer = signers.ExternalSigner(
    signing_cert=certs[0],
    cert_registry=cert_registry,
    signature_mechanism=None,
    signature_value=signature_value,
)

class FakeTimestamper(timestamps.TimeStamper):
    async def async_request_tsa_response(self, req):
        def task():
            return tsp.TimeStampResp.load(b64decode(timestamp_encoded))
        response = await to_thread(task)
        return response

signed_attrs = asyncio.run(
    signer.signed_attrs(data_digest, "sha256", use_pades=True)
)

sig_cms = signer.sign_prescribed_attributes(
    "sha256",
    signed_attrs=signed_attrs,
    timestamper=FakeTimestamper(include_nonce=False),
)

from time import time
start = time()
my_dump = sig_cms.dump()
end = time()
print(my_dump.hex())
print(start, end, end -start)
